"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Debugging_1 = require("./Debugging");
// Capture Math.random in case some other code swaps it out
const randomNumber = Math.random;
var ServiceRuntimeHelpers;
(function (ServiceRuntimeHelpers) {
    // Return a good enough unique string to identify request/response pairs across services
    function generateUniqueId() {
        return `${randomNumber()}`;
    }
    ServiceRuntimeHelpers.generateUniqueId = generateUniqueId;
    class StreamReader {
        constructor(method, helper) {
            this.method = method;
            this.helper = helper;
            this.hasAsyncIterator = false;
            this.cancel = () => __awaiter(this, void 0, void 0, function* () {
                throw new Error("cancel is not implemented");
            });
        }
        [Symbol.asyncIterator](argument) {
            if (this.hasAsyncIterator) {
                throw new Error("StreamReader.asyncIterator() may only be called once");
            }
            this.hasAsyncIterator = true;
            // The stream id will let the Router on the other end remember the corresponding iterator
            const method = this.method;
            const id = generateUniqueId();
            const logger = Debugging_1.ServiceDebugging.createLogger(`🏄‍ [${id}]`);
            return {
                next: () => __awaiter(this, void 0, void 0, function* () {
                    const result = yield this.helper({ method, argument, stream: { id, method: "next" } });
                    if (!isIteratorResult(result)) {
                        // FIXME: throw error instead of ending the iteration?
                        logger.log("StreamReader.next received an invalid iterator result for next()", result);
                        // Note: value = undefined is valid when done = true (contrary to what the types say)
                        return { done: true, value: undefined };
                    }
                    return result;
                }),
            };
        }
    }
    ServiceRuntimeHelpers.StreamReader = StreamReader;
    function isIteratorResult(result) {
        if (!result)
            return false;
        return result.done === true || (result.done === false && result.value !== undefined);
    }
})(ServiceRuntimeHelpers = exports.ServiceRuntimeHelpers || (exports.ServiceRuntimeHelpers = {}));
