"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const chalk_1 = require("chalk");
const validation_1 = require("./validation");
const defaultAPI = "https://api.framer.com";
exports.buildProjectConfig = argv => {
    // If a project path is specified, change process dir to that path
    // This is a workaround for a publish bug that causes builds
    // to fail when the script is run outside the project
    if (argv._[1]) {
        const projectPath = path.resolve(argv._[1]);
        validation_1.projectPathExists(projectPath);
        process.chdir(projectPath);
    }
    const config = {
        api: argv.api || defaultAPI,
        // Command is always run from inside project
        projectPath: process.cwd(),
        cachePath: "/tmp/framer",
        yarnLinkFolder: "/tmp/framer-yarn-link-folder",
        token: process.env.FRAMER_TOKEN,
    };
    return config;
};
exports.buildAuthenticationConfig = argv => {
    if (!argv._[1]) {
        throw new Error("An email address must be provided.");
    }
    return {
        api: argv.api || defaultAPI,
        email: argv._[1],
    };
};
exports.buildPublishConfig = argv => {
    if (Object.prototype.hasOwnProperty.call(argv, "yes") && typeof argv.yes !== "boolean") {
        throw new Error(chalk_1.default `The {bold --yes} argument can only be set as a flag.`);
    }
    if (Object.prototype.hasOwnProperty.call(argv, "new")) {
        if (typeof argv.new !== "string") {
            throw new Error(chalk_1.default `The {bold --new} argument can only be a string.`);
        }
        if (!argv.new.trim()) {
            throw new Error(chalk_1.default `The {bold --new} argument value cannot be blank.`);
        }
    }
    return {
        api: argv.api || defaultAPI,
        token: process.env.FRAMER_TOKEN,
        newName: argv.new,
        isNewArgSet: !!(argv.new && typeof argv.new === "string" && argv.new.trim()),
        isPublic: !!argv.public,
        isPrivate: !argv.public,
        disablePrompts: !!argv.yes,
        projectConfig: exports.buildProjectConfig(argv),
    };
};
