"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createDebugger = require("debug");
const interface_1 = require("./cli/interface");
const globalConfig = {
    output: "console",
};
exports.configure = ({ output = "console" }) => {
    globalConfig.output = output;
};
exports.createLogger = (prefix) => {
    /* tslint:disable:no-console */
    const { output } = globalConfig;
    // This is leaking CLI specific information into the
    // logger which is not ideal, but this wrapper makes it
    // significantly easier to see what is happening when
    // debugging the application, and both spinner and logging
    // can loosely be considered part of the interface
    const wrap = print => (...args) => {
        if (interface_1.spinner.isSpinning) {
            interface_1.spinner.stop();
            print(...args);
            interface_1.spinner.start();
        }
        else {
            print(...args);
        }
    };
    // If the logger is being used for debug, the logs will
    // be hidden by default - this might need to update in
    // the future so that information can be accessed
    // to provide a better experience for the user
    if (output === "debug") {
        const debug = createDebugger(prefix);
        // tslint:disable-next-line:no-console
        debug.log = console.log.bind(console);
        const errorDebug = createDebugger(prefix);
        const logger = debug.enabled ? wrap(debug) : debug;
        const errorLogger = errorDebug.enabled ? wrap(errorDebug) : errorDebug;
        return {
            log: logger,
            info: logger,
            warn: errorLogger,
            error: errorLogger,
        };
    }
    if (output === "silent") {
        const noop = () => null;
        return {
            log: noop,
            info: noop,
            warn: noop,
            error: noop,
        };
    }
    // If the logger is being used by a running server, the logs
    // must always be printed in order for the server
    // to properly communicate with the application
    return {
        log: wrap(console.log.bind(console)),
        info: wrap(console.info.bind(console)),
        warn: wrap(console.warn.bind(console)),
        error: wrap(console.error.bind(console)),
    };
    /* tslint:enable:no-console */
};
